"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@babel/core");
const utils_1 = require("./utils");
const constants_1 = require("./constants");
const rollup_plugin_terser_1 = require("rollup-plugin-terser");
const rollup_plugin_babel_1 = tslib_1.__importDefault(require("rollup-plugin-babel"));
const rollup_plugin_commonjs_1 = tslib_1.__importDefault(require("rollup-plugin-commonjs"));
const rollup_plugin_json_1 = tslib_1.__importDefault(require("rollup-plugin-json"));
const rollup_plugin_replace_1 = tslib_1.__importDefault(require("rollup-plugin-replace"));
const rollup_plugin_node_resolve_1 = tslib_1.__importDefault(require("rollup-plugin-node-resolve"));
const rollup_plugin_sourcemaps_1 = tslib_1.__importDefault(require("rollup-plugin-sourcemaps"));
const rollup_plugin_typescript2_1 = tslib_1.__importDefault(require("rollup-plugin-typescript2"));
const rollup_plugin_preserve_shebang_1 = tslib_1.__importDefault(require("@jaredpalmer/rollup-plugin-preserve-shebang"));
const replacements = [{ original: 'lodash', replacement: 'lodash-es' }];
const babelOptions = (format) => ({
    exclude: /node_modules/,
    extensions: [...core_1.DEFAULT_EXTENSIONS, 'ts', 'tsx'],
    plugins: [
        require.resolve('babel-plugin-annotate-pure-calls'),
        require.resolve('babel-plugin-dev-expression'),
        format !== 'cjs' && [
            require.resolve('babel-plugin-transform-rename-import'),
            { replacements },
        ],
    ].filter(Boolean),
});
function createRollupConfig(format, env, opts) {
    let shebang;
    return {
        // Tell Rollup the entry point to the package
        input: opts.input,
        // Tell Rollup which packages to ignore
        external: utils_1.external,
        // Establish Rollup output
        output: {
            // Set filenames of the consumer's package
            file: `${constants_1.paths.appDist}/${utils_1.safePackageName(opts.name)}.${format}.${env}.js`,
            // Pass through the file format
            format,
            // Do not let Rollup call Object.freeze() on namespace import objects
            // (i.e. import * as namespaceImportObject from...) that are accessed dynamically.
            freeze: false,
            // Do not let Rollup add a `__esModule: true` property when generating exports for non-ESM formats.
            esModule: false,
            // Rollup has treeshaking by default, but we can optimize it further...
            treeshake: {
                // We assume reading a property of an object never has side-effects.
                // This means tsdx WILL remove getters and setters on objects.
                //
                // @example
                //
                // const foo = {
                //  get bar() {
                //    console.log('effect');
                //    return 'bar';
                //  }
                // }
                //
                // const result = foo.bar;
                // const illegalAccess = foo.quux.tooDeep;
                //
                // Punchline....Don't use getters and setters
                propertyReadSideEffects: false,
            },
            name: opts.name || utils_1.safeVariableName(opts.name),
            sourcemap: true,
            globals: { react: 'React', 'react-native': 'ReactNative' },
            exports: 'named',
        },
        plugins: [
            rollup_plugin_node_resolve_1.default({
                mainFields: [
                    'module',
                    'main',
                    opts.target !== 'node' ? 'browser' : undefined,
                ].filter(Boolean),
            }),
            format === 'umd' &&
                rollup_plugin_commonjs_1.default({
                    // use a regex to make sure to include eventual hoisted packages
                    include: /\/node_modules\//,
                }),
            rollup_plugin_json_1.default(),
            rollup_plugin_typescript2_1.default({
                typescript: require('typescript'),
                cacheRoot: `./.rts2_cache_${format}`,
                tsconfigDefaults: {
                    compilerOptions: {
                        sourceMap: true,
                        declaration: true,
                        jsx: 'react',
                    },
                },
                tsconfigOverride: {
                    compilerOptions: {
                        target: 'esnext',
                    },
                },
            }),
            rollup_plugin_babel_1.default(babelOptions(format)),
            rollup_plugin_replace_1.default({
                'process.env.NODE_ENV': JSON.stringify(env),
            }),
            rollup_plugin_sourcemaps_1.default(),
            // sizeSnapshot({
            //   printInfo: false,
            // }),
            env === 'production' &&
                rollup_plugin_terser_1.terser({
                    sourcemap: true,
                    output: { comments: false },
                    compress: {
                        keep_infinity: true,
                        pure_getters: true,
                        collapse_vars: false,
                    },
                    ecma: 5,
                    toplevel: format === 'es' || format === 'cjs',
                    warnings: true,
                }),
            rollup_plugin_preserve_shebang_1.default({
                shebang,
            }),
        ],
    };
}
exports.createRollupConfig = createRollupConfig;
